var Client = require('node-rest-client').Client;
const BASE_URL = "https://api.kontist.com/api/";

/**
 * Calls rejecter when response has invalid status, calls resolver(data) if status is valid.
 * 
 * @param {*} data 
 * @param {*} response 
 * @param {*} resolver 
 * @param {*} rejecter 
 */
function handleResponse(data, response, resolver, rejecter) {
    if (response.statusCode < 200 || response.statusCode > 299) {
        console.error(response);
        rejecter(new Error(response.statusMessage));
    }
    resolver(data);
}

module.exports = class KontistClient extends Client {

    /**
     * Return information for logged in user. (via constructor {user: "", password: ""} parameters)
     */
    getUser() {
        return this.request("user");
    }

    /**
     * Return list of accounts.
     */
    getAccounts() {
        return this.request("accounts");
    }

    /**
     * Return list of transactions.
     * @param {number} accountId
     */
    getTransactions(accountId) {
        return this.request(`accounts/${accountId}/transactions`);
    }

    /**
     * Return list of transfers.
     * @param {number} accountId
     */
    getTransfers(accountId) {
        return this.request(`accounts/${accountId}/transfer`);
    }

    /**
     * Create a new transfer, needs to be confirmed with the ´confirmTransfer` method and a authorizationToken (you will receive via sms).
     * @param {number} accountId
     * @param {string} recipient
     * @param {string} iban
     * @param {number} amount cents
     * @param {string} note
     */
    initiateTransfer(accountId, recipient, iban, amount, note) {
        return this.request(`accounts/${accountId}/transfer`, "post", { recipient, iban, amount, note });
    }

    /**
     * 
     * @param {number} accountId
     * @param {string} transferId
     * @param {string} authorizationToken
     * @param {string} recipient
     * @param {string} iban
     * @param {number} amount cents
     * @param {string} note
     */
    confirmTransfer(accountId, transferId, authorizationToken, recipient, iban, amount, note) {
        return this.request(`accounts/${accountId}/transfer/${transferId}`, "put", { authorizationToken, recipient, iban, amount, note });
    }

    /**
     * Return a pdf statement.
     * @param {number} accountId
     */
    getStatement(accountId, year, month) {
        return this.request(`accounts/${accountId}/statements/${year}/${month}`);
    }

    /**
     * Helper to create promise with call to an endpoint.
     *
     * @param {string} endpoint 
     * @param {*} method = get
     * @param {*} data = null
     */
    request(endpoint, method, data) {
        const headers = {"Content-Type": "application/json"};
        return new Promise((resolve, reject) => {
            this[method || "get"](BASE_URL + endpoint, { data, headers }, (result, raw) => handleResponse(result, raw, resolve, reject))
                .on("error", (err) => reject(err));
        });
    }
}